extends Reference

# ==========================
# .API.gd
# 
# This file does NOT get loaded by the game. The game instead loads its own internal version of .API.gd
# containing all the same functions and members found in this file but with actual implimentations.
# This file exists only for documentation and auto-completion purposes. It does NOT need to be included in a mod you distribute.
# ==========================


# get Quasar's api version number (NOT the mod's version) as an int e.g. 200409
func get_api_version():
	return 1


# NONE is just a way to make -1 stand out. where relevant, the game's internal functions dont
# explicitly test for -1, they just test for <= 0.
const NONE = -1


# throughout this file, you'll see some _tester stuff. This is for integration into a rudimentary 
# testing script (.Tester.gd). Remember that all functions here are simply placeholders, and are
# overridden by the main game. The main game does not use _tester.
var _tester

# get_instance returns the instance of another script in the same mod folder.
# for example, if mods/default/ai.gd wants to reference a member or method of mods/default/library.gd,
# then ai.gd would call api.get_mod_script("library.gd")
# ignore the _tester stuff, this is all overridden by the main game
func get_instance(file_name_String, self_Reference):
	if _tester:
		return _tester.get_instance(file_name_String)

# like get_instance, except you can talk to foreign mods.. but the foreign mod script needs to have: var signature = 'abc123example'
func get_instance_by_signature(signature):
	return null

# note: calling popup_alert when a game is not running will instead call println and write to the mod logs
func popup_alert(msg_String):
	pass

# get/set_setting provide a way for the mod to save its configuration state in the sqlite save file.
# it's not intended to be called many times per tick.  for high performance or more complexity,
# the mod should instead create/update its own tables. (which is not yet supported)
func set_setting(signature, setting_name, value):
	pass

func get_setting(signature, setting_name):
	return null

# print a line to the mod loader window's text output
func println(msg_String = ""):
	if _tester:
		_tester.get_node('Console').text += str(msg_String) + "\n"


# run an SQL query on the database. query results are returned as an array, one element for each row.
# each element is a Dictionary, where the keys are the column names of the query.
#
# example usage: 
# var data = api.fetch("select ShipName from Ship where ShipID = %s" % ship_id)
# return data[0]['ShipName'] if !data.empty() else api.NONE
func fetch(sql_String):
	return []

# analagous to Fleet Orders > Special Orders/Organization tab > Move ships between task groups
# example:
# api.move_ships_to_fleet(12345, "66,67,78", context)
func move_ships_to_fleet(fleet_id, ship_id_list, context):
	pass

# delete all fleets belonging to context['RaceID'] containing no ships
func delete_empty_fleets(context):
	pass

# load PNG or JPG (determined by the filename's extension) and return a Texture object
# file_name is relative to the mod's folder.  if the image is at mods/coolmod/icons/test.png, then you would pass "icons/test.png"
func load_texture(file_name_String, self_Reference): 
	return null

# get a node provided an absolute node path, for example "/root/Main/Game/SystemMap/SystemMapButtons/Commanders"
func get_node(node_path):
	return null
	
# akin to pressing Refresh All in the game
func full_refresh():
	pass

# returns absolute filesystem path to the given mod, which is either just 'self', or is an object that was received by get_instance_by_signature()
# an example return string would be "C:\Quasar4x/mods/examples/"
func get_path_to_mod(mod_Reference):
	return ""

# make a string containing a comma separated list of values for one specific column name in the result of an api.fetch() call
# example:
# var ships = api.fetch("select ShipID from Ships where FleetID = 1234")
# var ships_list = api.column_to_list(ships, "ShipID")  # ships_list is now "66,67,78"
func column_to_list(_rows, _col_name):
	return ""
	

# execute() writes to the db, e.g., run an insert, update, delete, create, etc..
#
# This function is provided in case you want to use the database to store custom AI logic, 
# or if you want to manipulate the DB in a way that the API doesn't yet have a function for, and you've read the warning below.
#
# WARNING: THIS FUNCTION SHOULD NEVER BE USED
# even if you think you know what you're doing, unless you are messing with tables YOU created, 
# you are probably forgetting a side effect or two.  You REALLY shouldn't use this to mess with 
# data unless its your OWN PRIVATE tables. 
#
func execute(sql_String):
	pass
	

# returns the integer result of the SQL query 'select changes()', which is how many rows were affected by the previous SQL
func changes():
	return 0

# returns the integer result of the SQL query 'select last_insert_rowid()', which is the rowid (the primary key) assigned to the most recent successful insert
func get_last_insert_id():
	return -1

	
# These are the signals you can connect to in your script's on_program_load() func.
# every callback receives at least one parameter, called the chalkboard, which can be ignored, and is just provided
# as a temporary cache of data that can be used to transfer data between different scripts receiving the same callback.
# here is an example of what you could have in your script:   (also see footnote at the bottom of this file)
# 
#
#	func on_program_load():
#		api.connect_callback_test(self)
#
#	func on_callback_test(chalkboard, _context):
#		api.println("chalkboard contains: " + to_json(chalkboard))

# on_callback_test is called when the Test button is pressed on the mod list in game
# Two parameters:    chalkboard: Dictionary (see above), sample_data: Dictionary
# ignore the _tester stuff, this is all overridden by the main game

func connect_callback_test(listener, func_name = "on_callback_test"):
	if _tester:
		_tester.connect_callback_test(listener, func_name)
		
# this is called every subpulse after player fleet training moves are created and before conditional orders are checked
func connect_generate_npr_moves(listener, func_name = "on_generate_npr_moves"):
	pass



# normally, your callback function shouldn't return anything. But if you want, you can return one of these:
class Request:
	enum { MoveMeToEnd }
# see mods/default/precursors.gd for an example

# ----------
# This file is self documenting as much as possible. Parameters are named:  their_purpose_TheirType
# (GDScript does have a static-typing feature but I opted against it.)
# Note that the game will never load this file, it actually loads its own internal .API.gd script 
# which contains the same signals and functions above but with actual implementations.
